#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <pthread.h>
#include <glib.h>
#include <gtk/gtk.h>
#include <xmms/plugin.h>
#include <xmms/util.h>
#include <xmms/configfile.h>
#include <libsap.h>

#include "fmtstr.h"
#include "sap.h"
#include "insap.h"
#include "insap_fileinfo.h"
#include "insap_about.h"
#include "insap_config.h"

struct insap_config insap_cfg;

static void loadconfig()
{
	ConfigFile *cfg;
	gchar *fn;
	gint i;
	gboolean b;
	gchar *s;

	insap_cfg.usenameformat = TRUE;
	insap_cfg.nameformat = g_strdup("%n [%s]");
	insap_cfg.silencedetector = FALSE;
	insap_cfg.silencedetectorsecs = 3;
	insap_cfg.detectbycontent = FALSE;

	fn = g_strconcat(g_get_home_dir(), "/.xmms/config", NULL);

	#ifdef __DEBUG__
	MESGB(__FILE__,__LINE__) "LOADCONFIG: %s\n",fn MESGE
	#endif
	
	cfg = xmms_cfg_open_file(fn);

	if (cfg) {
		if ( xmms_cfg_read_boolean(cfg, "insap", "use_name_format", &b) )       insap_cfg.usenameformat = b;
		if ( xmms_cfg_read_string(cfg,  "insap", "name_format", &s) )          insap_cfg.nameformat = s;
		if ( xmms_cfg_read_boolean(cfg, "insap", "silence_detector", &b) )     insap_cfg.silencedetector = b;
		if ( xmms_cfg_read_int(cfg,     "insap", "silence_detector_secs", &i) ) insap_cfg.silencedetectorsecs = i;
		if ( xmms_cfg_read_boolean(cfg, "insap", "detect_by_content", &b) )     insap_cfg.detectbycontent = b;
		
		xmms_cfg_free(cfg);
	}
	g_free(fn);
}

static void saveconfig()
{
	ConfigFile *cfg;
	gchar *fn;
	
	fn = g_strconcat(g_get_home_dir(), "/.xmms/config", NULL);

	#ifdef __DEBUG__
	MESGB(__FILE__,__LINE__) "SAVECONFIG: %s\n",fn MESGE
	#endif
	
	cfg = xmms_cfg_open_file(fn);
	if (!cfg) cfg = xmms_cfg_new();

	xmms_cfg_write_boolean(cfg, "insap", "use_name_format", insap_cfg.usenameformat);
	xmms_cfg_write_string(cfg,  "insap", "name_format", insap_cfg.nameformat);
	xmms_cfg_write_boolean(cfg, "insap", "silence_detector", insap_cfg.silencedetector);
	xmms_cfg_write_int(cfg,     "insap", "silence_detector_secs", insap_cfg.silencedetectorsecs);
	xmms_cfg_write_boolean(cfg, "insap", "detect_by_content", insap_cfg.detectbycontent);
		
	xmms_cfg_write_file(cfg, fn);
	xmms_cfg_free(cfg);
	g_free(fn);
}

static struct fmt_option fmt_opts[] = {
	{ 'n', NULL },
	{ 'a', NULL },
	{ 'd', NULL },
	{ 's', NULL },
	{ 0, 0 }
};
enum { insap_nameopt, insap_authoropt, insap_dateopt, insap_songsopt };

static void insap_init(void);
static int  insap_isourfile(gchar *filename);
static void insap_playfile(gchar *filename);
static void insap_stop(void);
static void insap_pause(short p);
static void insap_seek(int time);
static int  insap_gettime(void);
static void insap_cleanup(void);
static void insap_getsonginfo(gchar *filename, gchar **title, int *length);

static int songsNum = -1;
static int currentSong = -1;

static gboolean going;
static gboolean audio_error;
static pthread_t play_thread;
static gboolean paused;

InputPlugin sap_ip =
{
	NULL,                          /* void *handle; */
	NULL,                          /* char *filename; */
	INSAP_STR " " INSAP_VER,       /* char *description; */
	insap_init,                    /* void (*init) (void); */
	insap_about,                   /* void (*about) (void); */
	insap_config,                  /* void (*configure) (void); */
	insap_isourfile,               /* int (*is_our_file) (char *filename); */
	NULL,                          /* GList *(*scan_dir) (char *dirname); */
	insap_playfile,                /* void (*play_file) (char *filename); */
	insap_stop,                    /* void (*stop) (void); */
	insap_pause,                   /* void (*pause) (short paused); */
	insap_seek,                    /* void (*seek) (int time); */
	NULL,                          /* void (*set_eq) (int on, float preamp, float *bands); */
	insap_gettime,                 /* int (*get_time) (void); */
	NULL,                          /* void (*get_volume) (int *l, int *r); */
	NULL,                          /* void (*set_volume) (int l, int r); */
	insap_cleanup,                 /* void (*cleanup) (void); */
	NULL,                          /* InputVisType (*get_vis_type) (void); */
	NULL,                          /* void (*add_vis_pcm) (int time, AFormat fmt, int nch, int length, void *ptr); */
	NULL,                          /* void (*set_info) (char *title, int length, int rate, int freq, int nch); */
	NULL,                          /* void (*set_info_text) (char *text); */
	insap_getsonginfo,             /* void (*get_song_info) (char *filename, char **title, int *length); */
	insap_fileinfo,                /* void (*file_info_box) (char *filename); */
	NULL                           /* OutputPlugin *output; */
};

static sapMUSICstrc *currentFile;

InputPlugin *get_iplugin_info(void)
{
	#ifdef __DEBUG__
	MESGB(__FILE__,__LINE__) "GET_IPLUGIN_INFO\n" MESGE
	#endif
	
	return &sap_ip;
}

static void insap_init(void)
{
	#ifdef __DEBUG__
	MESGB(__FILE__,__LINE__) "INIT\n" MESGE
	#endif
	
	loadconfig();
}

static void insap_cleanup(void)
{
	#ifdef __DEBUG__
	MESGB(__FILE__,__LINE__) "CLEANUP\n" MESGE
	#endif
	
	saveconfig();
}

static int insap_isourfile(gchar *filename)
{
	int l = strlen(filename);
	
	#ifdef __DEBUG__
	MESGB(__FILE__,__LINE__) "IS OUR FILE: %s\n",filename MESGE
	#endif
	
	if (insap_cfg.detectbycontent) return isSapFile(filename);
	else return l>4 && strncasecmp(&filename[l-4], ".sap", 4)==0;
}

#define BUF_SAMPLES 8128
#define BUF_BYTES BUF_SAMPLES * 2

static void *play_loop(void *arg)
{
	gint16 playBuffer[BUF_SAMPLES];

	while (going) {
		while (paused) xmms_usleep(30000);
		sapRenderBuffer(playBuffer, BUF_SAMPLES/2);
		
		// silence detector - changes song if silence is longer than 'silencetime' seconds
		if (insap_cfg.silencedetector) {
			static long silencecounter = 0;
			int silence = 1;
			int i;
			for (i=0; i<BUF_SAMPLES; i++) if ((playBuffer[i]&0xffff)!=SILENCE_WORD) silence=0;
			if (silence) silencecounter+=BUF_SAMPLES; else silencecounter=0;
			if (silencecounter >= 44100*insap_cfg.silencedetectorsecs) {
				currentSong = (currentSong+1) % (songsNum+1);
				sapPlaySong(currentSong);
			}
		}
		
//		sap_ip.add_vis_pcm(sap_ip.output->written_time(), FMT_S16_LE, 2, BUF_BYTES, playBuffer);
		while (sap_ip.output->buffer_free() < BUF_BYTES && going) xmms_usleep(30000);
		if (going) sap_ip.output->write_audio(playBuffer, BUF_BYTES);	
	}

	sap_ip.output->buffer_free();
	sap_ip.output->buffer_free();
	pthread_exit(NULL);
}

static void insap_playfile(gchar *filename)
{
	gchar name[256];
	// (char *title, int length, int rate, int freq, int nch);

	#ifdef __DEBUG__
	MESGB(__FILE__,__LINE__) "PLAY FILE: %s\n",filename MESGE
	#endif
	
	paused = FALSE;
	going = TRUE;
	audio_error = FALSE;
	if (sap_ip.output->open_audio(FMT_S16_LE, 44100, 2) == 0) {
		audio_error = TRUE;
		going = FALSE;
		return;
	}
	
	currentFile = sapLoadMusicFile( filename );
	if (currentFile==NULL) {
		sap_ip.set_info("ERROR", 0, 0, 0, 1);
		going = FALSE;
		return;
	}
	songsNum = currentFile->numOfSongs - 1;
	currentSong = currentFile->defSong;

	if (insap_cfg.usenameformat) {
		readSapComment(filename);
		setSapCommentArray(NULL);
		fmt_opts[insap_nameopt].str   = getSapCommentElem("NAME");
		fmt_opts[insap_authoropt].str = getSapCommentElem("AUTHOR");
		fmt_opts[insap_dateopt].str   = getSapCommentElem("DATE");
		fmt_opts[insap_songsopt].str  = getSapCommentElem("SONGS");
		if (strlen(fmt_opts[insap_songsopt].str)==0) fmt_opts[insap_songsopt].str = "1";
		fmtstr(name, insap_cfg.nameformat, fmt_opts);
	}
	else strcpy(name, g_basename(filename));
	sap_ip.set_info(name, (songsNum>0)?songsNum * 1000:-1, 0, 44100, 2);
	
	pthread_create(&play_thread, NULL, play_loop, NULL);
}

static void insap_stop(void)
{
	#ifdef __DEBUG__
	MESGB(__FILE__,__LINE__) "STOP\n" MESGE
	#endif
	
	if (going) {
		going = FALSE;
		pthread_join(play_thread, NULL);
		sap_ip.output->close_audio();
	}
}

static void insap_pause(short p)
{
	#ifdef __DEBUG__
	MESGB(__FILE__,__LINE__) "PAUSE: %d\n",p MESGE
	#endif
	
	paused = p;
	sap_ip.output->pause(p);
}

static void insap_seek(int time)
{
	#ifdef __DEBUG__
	MESGB(__FILE__,__LINE__) "SEEK: %d\n",time MESGE
	#endif
	
	if (currentSong == time) return;
	
	currentSong = time;
	
	paused = TRUE;
	sapPlaySong(currentSong);
	sap_ip.output->flush(currentSong * 1000);
	paused = FALSE;
}

static int insap_gettime(void)
{
	#ifdef __DEBUG__
//	MESGB(__FILE__,__LINE__) "GET TIME\n" MESGE
	#endif

//	char buf[128];
//	static int i = 0;
//	sprintf(buf,"%05d",i);
//	plugin_ip.set_info_text(buf);
//	i=(i+100)%(songsNum*1000);

//	return sap_ip.output->output_time();

	return (currentSong>0)?currentSong * 1000:0;
}

static void insap_getsonginfo(gchar *filename, gchar **title, int *len)
{
	char buf[1024];
	char *p;
	
	#ifdef __DEBUG__
	MESGB(__FILE__,__LINE__) "GET SONG INFO: %s\n",filename MESGE
	#endif

	readSapComment(filename);
	setSapCommentArray(NULL);
		
	if (insap_cfg.usenameformat) {
		fmt_opts[insap_nameopt].str   = getSapCommentElem("NAME");
		fmt_opts[insap_authoropt].str = getSapCommentElem("AUTHOR");
		fmt_opts[insap_dateopt].str   = getSapCommentElem("DATE");
		fmt_opts[insap_songsopt].str  = getSapCommentElem("SONGS");
		if (strlen(fmt_opts[insap_songsopt].str)==0) fmt_opts[insap_songsopt].str = "1";
		fmtstr(buf, insap_cfg.nameformat, fmt_opts);
		*title = strdup(buf);
	}
	else *title = strdup(g_basename(filename));

//	strcpy(buf, getSapCommentElem("AUTHOR"));
//	if (strlen(buf)>0) strcat(buf, " - ");
//	strcat(buf, getSapCommentElem("NAME"));
//	if (strlen(buf)>0) *title = strdup(buf);
//	else *title = NULL;
	
	p = getSapCommentElem("SONGS");
	if (p==NULL || atoi(p)<2) *len = -1;
	else *len = (atoi(p)-1) * 1000;
}
