#include <string.h>
#include <glib.h>
#include <gtk/gtk.h>
#include "insap.h"
#include "sap.h"
#include "insap_config.h"

extern struct insap_config insap_cfg;

static GtkWidget *config_win = NULL,
		 *entry_nameformat,
		 *cbox_nameformat,
		 *lab1_nameformat,
		 *lab2_nameformat,
		 *cbox_silencedetector,
		 *sbtn_silencedetector,
		 *cbox_detectbycontent;

static void cb_cbox_nameformat(GtkWidget *wid, char *data)
{
	gboolean active;
	active = GTK_TOGGLE_BUTTON(wid)->active;
	
	#ifdef __DEBUG__
	MESGB(__FILE__,__LINE__) "CALLBACK (cbox_nameformat): %s\n",active?"ON":"OFF" MESGE
	#endif	
	
	gtk_widget_set_sensitive(entry_nameformat, active);
	gtk_widget_set_sensitive(lab1_nameformat, active);
	gtk_widget_set_sensitive(lab2_nameformat, active);
	
	insap_cfg.usenameformat = active;
}

static void cb_entry_nameformat(GtkWidget *wid, char *data)
{
	gchar *text;
	text = gtk_editable_get_chars(GTK_EDITABLE(wid), 0, -1);

	#ifdef __DEBUG__
	MESGB(__FILE__,__LINE__) "CALLBACK (entry_nameformat): %s\n",text MESGE
	#endif	

	g_free(insap_cfg.nameformat);
	insap_cfg.nameformat = text;
}

static void cb_cbox_silencedetector(GtkWidget *wid, char *data)
{
	gboolean active;
	active = GTK_TOGGLE_BUTTON(wid)->active;

	#ifdef __DEBUG__
	MESGB(__FILE__,__LINE__) "CALLBACK (cbox_silencedetector): %s\n",active?"ON":"OFF" MESGE
	#endif	
	
	insap_cfg.silencedetector = active;
	gtk_widget_set_sensitive(sbtn_silencedetector, active);
}

static void cb_sbtn_silencedetector(GtkWidget *wid, char *data)
{
	gint secs;
	secs = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(wid));

	#ifdef __DEBUG__
	MESGB(__FILE__,__LINE__) "CALLBACK (sbtn_silencedetector): %d\n",secs MESGE
	#endif	
	
	insap_cfg.silencedetectorsecs = secs;
}

static void cb_detectbycontent(GtkWidget *wid, char *data)
{
	gboolean active;
	active = GTK_TOGGLE_BUTTON(wid)->active;

	#ifdef __DEBUG__
	MESGB(__FILE__,__LINE__) "CALLBACK (cbox_detectbycontent): %s\n",active?"ON":"OFF" MESGE
	#endif	

	insap_cfg.detectbycontent = active;
}

void insap_config(void)
{
	GtkWidget *vbox, *vbox1, *vbox2,
		  *frame;

	#ifdef __DEBUG__
	MESGB(__FILE__,__LINE__) "ABOUT\n" MESGE
	#endif

	if (config_win) return;
	config_win = gtk_window_new(GTK_WINDOW_DIALOG);
	gtk_signal_connect(GTK_OBJECT(config_win), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &config_win);
	
	gtk_window_set_title(GTK_WINDOW(config_win), INSAP_STR " Config");
	gtk_window_set_policy(GTK_WINDOW(config_win), FALSE, FALSE, FALSE);
	gtk_window_set_position(GTK_WINDOW(config_win), GTK_WIN_POS_NONE);
	gtk_container_border_width(GTK_CONTAINER(config_win), 3);

	vbox = gtk_vbox_new(FALSE, 2);
	gtk_container_add(GTK_CONTAINER(config_win), vbox);


	frame = gtk_frame_new("Title");
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
	vbox1 = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(frame), vbox1);
	gtk_container_set_border_width(GTK_CONTAINER(vbox1), 5);

	cbox_nameformat = gtk_check_button_new_with_label("Use sap comment");
	gtk_box_pack_start(GTK_BOX(vbox1), cbox_nameformat, FALSE, FALSE, 0);

	entry_nameformat = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(vbox1), entry_nameformat, FALSE, FALSE, 0);
	
	vbox2 = gtk_hbox_new(TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox1), vbox2, FALSE, FALSE, 0);	
	lab1_nameformat = gtk_label_new(
	"%n - name\n"
	"%a - author");
	gtk_label_set_justify(GTK_LABEL(lab1_nameformat), GTK_JUSTIFY_LEFT);
	gtk_box_pack_start(GTK_BOX(vbox2), lab1_nameformat, FALSE, FALSE, 0);	
	lab2_nameformat = gtk_label_new(	
	"%d - date\n"
	"%s - songs");
	gtk_label_set_justify(GTK_LABEL(lab2_nameformat), GTK_JUSTIFY_LEFT);
	gtk_box_pack_start(GTK_BOX(vbox2), lab2_nameformat, FALSE, FALSE, 0);
	

	frame = gtk_frame_new("Silence detector");
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
	vbox1 = gtk_vbox_new(TRUE, 0);
	gtk_container_add (GTK_CONTAINER(frame), vbox1);
	gtk_container_set_border_width(GTK_CONTAINER(vbox1), 5);

	cbox_silencedetector = gtk_check_button_new_with_label("Enable silence detector");
	gtk_box_pack_start(GTK_BOX(vbox1), cbox_silencedetector, FALSE, FALSE, 0);
	
 	sbtn_silencedetector = gtk_spin_button_new(GTK_ADJUSTMENT(gtk_adjustment_new (3, 1, 30, 1, 10, 10)), 1, 0);
	gtk_box_pack_start(GTK_BOX(vbox1), sbtn_silencedetector, FALSE, FALSE, 0);

	
	frame = gtk_frame_new("Options");
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
	vbox1 = gtk_vbox_new(TRUE, 0);
	gtk_container_add (GTK_CONTAINER(frame), vbox1);

	cbox_detectbycontent = gtk_check_button_new_with_label("Detect files by content (instead of the extension)");
	gtk_container_add (GTK_CONTAINER(vbox1), cbox_detectbycontent);


	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cbox_nameformat), insap_cfg.usenameformat);
	gtk_entry_set_text(GTK_ENTRY(entry_nameformat), insap_cfg.nameformat);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cbox_silencedetector), insap_cfg.silencedetector);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(sbtn_silencedetector), insap_cfg.silencedetectorsecs);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cbox_detectbycontent), insap_cfg.detectbycontent);
	cb_cbox_nameformat(cbox_nameformat, NULL);
	cb_cbox_silencedetector(cbox_silencedetector, NULL);

	gtk_widget_show_all(config_win);
	
	gtk_signal_connect(GTK_OBJECT(cbox_nameformat), "toggled", GTK_SIGNAL_FUNC(cb_cbox_nameformat), NULL);
	gtk_signal_connect(GTK_OBJECT(entry_nameformat), "focus_out_event", GTK_SIGNAL_FUNC(cb_entry_nameformat), NULL);
	gtk_signal_connect(GTK_OBJECT(cbox_silencedetector), "toggled", GTK_SIGNAL_FUNC(cb_cbox_silencedetector), NULL);
	gtk_signal_connect(GTK_OBJECT(sbtn_silencedetector), "focus_out_event", GTK_SIGNAL_FUNC(cb_sbtn_silencedetector), NULL);
	gtk_signal_connect(GTK_OBJECT(cbox_detectbycontent), "toggled", GTK_SIGNAL_FUNC(cb_detectbycontent), NULL);
//	gtk_signal_connect(GTK_OBJECT(entry_nameformat), "changed", GTK_SIGNAL_FUNC(cb_entry_nameformat), NULL);
}
